@@ -16,51 +16,56 @@
 */
 
 #include "BarcodeFormat.h"
-#include "BitHacks.h"
+
 #include "ZXContainerAlgorithms.h"
 
 #include <algorithm>
 #include <cctype>
 #include <iterator>
-#include <string>
 #include <sstream>
 #include <stdexcept>
+#include <string>
 
 namespace ZXing {
 
-static const char* FORMAT_STR[] = {
-	"NONE",
-	"AZTEC",
-	"CODABAR",
-	"CODE_39",
-	"CODE_93",
-	"CODE_128",
-	"DATA_MATRIX",
-	"EAN_8",
-	"EAN_13",
-	"ITF",
-	"MAXICODE",
-	"PDF_417",
-	"QR_CODE",
-	"RSS_14",
-	"RSS_EXPANDED",
-	"UPC_A",
-	"UPC_E",
-	"UPC_EAN_EXTENSION",
+struct BarcodeFormatName
+{
+	BarcodeFormat format;
+	const char* name;
 };
 
-static_assert(Size(FORMAT_STR) == (int)BarcodeFormats::bitIndex(BarcodeFormat::_max) + 1,
-			  "FORMAT_STR array is out of sync with BarcodeFormat");
+static BarcodeFormatName NAMES[] = {
+	{BarcodeFormat::None, "None"},
+	{BarcodeFormat::Aztec, "Aztec"},
+	{BarcodeFormat::Codabar, "Codabar"},
+	{BarcodeFormat::Code39, "Code39"},
+	{BarcodeFormat::Code93, "Code93"},
+	{BarcodeFormat::Code128, "Code128"},
+	{BarcodeFormat::DataBar, "DataBar"},
+	{BarcodeFormat::DataBarExpanded, "DataBarExpanded"},
+	{BarcodeFormat::DataMatrix, "DataMatrix"},
+	{BarcodeFormat::EAN8, "EAN-8"},
+	{BarcodeFormat::EAN13, "EAN-13"},
+	{BarcodeFormat::ITF, "ITF"},
+	{BarcodeFormat::MaxiCode, "MaxiCode"},
+	{BarcodeFormat::PDF417, "PDF417"},
+	{BarcodeFormat::QRCode, "QRCode"},
+	{BarcodeFormat::UPCA, "UPC-A"},
+	{BarcodeFormat::UPCE, "UPC-E"},
+	{BarcodeFormat::OneDCodes, "1D-Codes"},
+	{BarcodeFormat::TwoDCodes, "2D-Codes"},
+};
 
 const char* ToString(BarcodeFormat format)
 {
-	return FORMAT_STR[BarcodeFormats::bitIndex(format)];
+	auto i = FindIf(NAMES, [format](auto& v) { return v.format == format; });
+	return i == std::end(NAMES) ? nullptr : i->name;
 }
 
 std::string ToString(BarcodeFormats formats)
 {
-	if (formats.testFlag(BarcodeFormat::NONE))
-		return ToString(BarcodeFormat::NONE);
+	if (formats.empty())
+		return ToString(BarcodeFormat::None);
 	std::string res;
 	for (auto f : formats)
 		res += ToString(f) + std::string("|");
@@ -70,16 +75,14 @@ std::string ToString(BarcodeFormats formats)
 static std::string NormalizeFormatString(std::string str)
 {
 	std::transform(str.begin(), str.end(), str.begin(), [](char c) { return (char)std::tolower(c); });
-	str.erase(std::remove(str.begin(), str.end(), '_'), str.end());
+	str.erase(std::remove_if(str.begin(), str.end(), [](char c) { return Contains("_-[]", c); }), str.end());
 	return str;
 }
 
 static BarcodeFormat ParseFormatString(const std::string& str)
 {
-	auto pos = std::find_if(std::begin(FORMAT_STR), std::end(FORMAT_STR),
-							[str](auto fmt) { return NormalizeFormatString(fmt) == str; });
-	return pos == std::end(FORMAT_STR) ? BarcodeFormat::NONE
-									   : BarcodeFormat(1 << (std::distance(std::begin(FORMAT_STR), pos) - 1));
+	auto i = FindIf(NAMES, [str](auto& v) { return NormalizeFormatString(v.name) == str; });
+	return i == std::end(NAMES) ? BarcodeFormat::None : i->format;
 }
 
 BarcodeFormat BarcodeFormatFromString(const std::string& str)
@@ -97,7 +100,7 @@ BarcodeFormats BarcodeFormatsFromString(const std::string& str)
 	for (std::string token; std::getline(input, token, '|');) {
 		if(!token.empty()) {
 			auto bc = ParseFormatString(token);
-			if (bc == BarcodeFormat::NONE)
+			if (bc == BarcodeFormat::None)
 				throw std::invalid_argument("This is not a valid barcode format: " + token);
 			res |= bc;
 		}
