@@ -17,15 +17,18 @@
 */
 
 #include "DMDetector.h"
+
 #include "BitMatrix.h"
 #include "BitMatrixCursor.h"
+#include "ByteMatrix.h"
 #include "DetectorResult.h"
-#include "ResultPoint.h"
 #include "GridSampler.h"
-#include "Point.h"
-#include "WhiteRectDetector.h"
-
 #include "LogMatrix.h"
+#include "Point.h"
+#include "RegressionLine.h"
+#include "ResultPoint.h"
+#include "Scope.h"
+#include "WhiteRectDetector.h"
 
 #include <algorithm>
 #include <array>
@@ -37,8 +40,7 @@
 #include <utility>
 #include <vector>
 
-namespace ZXing {
-namespace DataMatrix {
+namespace ZXing::DataMatrix {
 
 /**
 * The following code is the 'old' code by Sean Owen based on the Java upstream project.
@@ -103,13 +105,13 @@ static ResultPointsAndTransitions TransitionsBetween(const BitMatrix& image, con
 	return ResultPointsAndTransitions{ &from, &to, transitions };
 }
 
-inline static bool IsValidPoint(const ResultPoint& p, int imgWidth, int imgHeight)
+static bool IsValidPoint(const ResultPoint& p, int imgWidth, int imgHeight)
 {
 	return p.x() >= 0 && p.x() < imgWidth && p.y() > 0 && p.y() < imgHeight;
 }
 
 template <typename T>
-inline static float RoundToNearestF(T x)
+static float RoundToNearestF(T x)
 {
 	return static_cast<float>(std::round(x));
 }
@@ -258,23 +260,21 @@ static void OrderByBestPatterns(const ResultPoint*& p0, const ResultPoint*& p1,
 static DetectorResult DetectOld(const BitMatrix& image)
 {
 	ResultPoint pointA, pointB, pointC, pointD;
-	if (!WhiteRectDetector::Detect(image, pointA, pointB, pointC, pointD)) {
+	if (!DetectWhiteRect(image, pointA, pointB, pointC, pointD)) {
 		return {};
 	}
 
 	// Point A and D are across the diagonal from one another,
 	// as are B and C. Figure out which are the solid black lines
 	// by counting transitions
-	std::array<ResultPointsAndTransitions, 4> transitions = {
+	std::array transitions = {
 		TransitionsBetween(image, pointA, pointB),
 		TransitionsBetween(image, pointA, pointC),
 		TransitionsBetween(image, pointB, pointD),
 		TransitionsBetween(image, pointC, pointD),
 	};
 	std::sort(transitions.begin(), transitions.end(),
-			  [](const ResultPointsAndTransitions& a, const ResultPointsAndTransitions& b) {
-				  return a.transitions < b.transitions;
-			  });
+			  [](const auto& a, const auto& b) { return a.transitions < b.transitions; });
 
 	// Sort by number of transitions. First two will be the two solid sides; last two
 	// will be the two alternating black/white sides
@@ -292,17 +292,17 @@ static DetectorResult DetectOld(const BitMatrix& image)
 	const ResultPoint* bottomRight = nullptr;
 	const ResultPoint* bottomLeft = nullptr;
 	const ResultPoint* topLeft = nullptr;
-	for (const auto& entry : pointCount) {
-		if (entry.second == 2) {
-			bottomLeft = entry.first; // this is definitely the bottom left, then -- end of two L sides
+	for (const auto& [point, count] : pointCount) {
+		if (count == 2) {
+			bottomLeft = point; // this is definitely the bottom left, then -- end of two L sides
 		}
 		else {
 			// Otherwise it's either top left or bottom right -- just assign the two arbitrarily now
 			if (bottomRight == nullptr) {
-				bottomRight = entry.first;
+				bottomRight = point;
 			}
 			else {
-				topLeft = entry.first;
+				topLeft = point;
 			}
 		}
 	}
@@ -359,7 +359,7 @@ static DetectorResult DetectOld(const BitMatrix& image)
 
 	ResultPoint correctedTopRight;
 
-	// Rectanguar symbols are 6x16, 6x28, 10x24, 10x32, 14x32, or 14x44. If one dimension is more
+	// Rectangular symbols are 6x16, 6x28, 10x24, 10x32, 14x32, or 14x44. If one dimension is more
 	// than twice the other, it's certainly rectangular, but to cut a bit more slack we accept it as
 	// rectangular if the bigger side is at least 7/4 times the other:
 	if (4 * dimensionTop >= 7 * dimensionRight || 4 * dimensionRight >= 7 * dimensionTop) {
@@ -398,7 +398,7 @@ static DetectorResult DetectOld(const BitMatrix& image)
 			dimensionCorrected++;
 		}
 
-		dimensionTop = dimensionRight = dimension;
+		dimensionTop = dimensionRight = dimensionCorrected;
 	}
 
 	return SampleGrid(image, *topLeft, *bottomLeft, *bottomRight, correctedTopRight, dimensionTop, dimensionRight);
@@ -409,41 +409,11 @@ static DetectorResult DetectOld(const BitMatrix& image)
 * It is performing something like a (back) trace search along edges through the bit matrix, first looking for
 * the 'L'-pattern, then tracing the black/white borders at the top/right. Advantages over the old code are:
 *  * works with lower resolution scans (around 2 pixel per module), due to sub-pixel precision grid placement
-*  * works with real-world codes that have just one module wide quite-zone (which is perfectly in spec)
+*  * works with real-world codes that have just one module wide quiet-zone (which is perfectly in spec)
 */
 
-class RegressionLine
+class DMRegressionLine : public RegressionLine
 {
-	std::vector<PointF> _points;
-	PointF _directionInward;
-	double a = NAN, b = NAN, c = NAN;
-
-	friend PointF intersect(const RegressionLine& l1, const RegressionLine& l2);
-
-	bool evaluate(const std::vector<PointF>& ps)
-	{
-		auto mean = std::accumulate(ps.begin(), ps.end(), PointF()) / ps.size();
-		double sumXX = 0, sumYY = 0, sumXY = 0;
-		for (auto& p : ps) {
-			sumXX += (p.x - mean.x) * (p.x - mean.x);
-			sumYY += (p.y - mean.y) * (p.y - mean.y);
-			sumXY += (p.x - mean.x) * (p.y - mean.y);
-		}
-		if (sumYY >= sumXX) {
-			a = +sumYY / std::sqrt(sumYY * sumYY + sumXY * sumXY);
-			b = -sumXY / std::sqrt(sumYY * sumYY + sumXY * sumXY);
-		} else {
-			a = +sumXY / std::sqrt(sumXX * sumXX + sumXY * sumXY);
-			b = -sumXX / std::sqrt(sumXX * sumXX + sumXY * sumXY);
-		}
-		if (dot(_directionInward, normal()) < 0) {
-			a = -a;
-			b = -b;
-		}
-		c = dot(normal(), mean); // (a*mean.x + b*mean.y);
-		return dot(_directionInward, normal()) > 0.5; // angle between original and new direction is at most 60 degree
-	}
-
 	template <typename Container, typename Filter>
 	static double average(const Container& c, Filter f)
 	{
@@ -458,51 +428,15 @@ class RegressionLine
 	}
 
 public:
-	const std::vector<PointF>& points() const { return _points; }
-	int length() const { return _points.size() >= 2 ? int(distance(_points.front(), _points.back())) : 0; }
-	bool isValid() const { return !std::isnan(a); }
-	PointF normal() const { return isValid() ? PointF(a, b) : _directionInward; }
-	double signedDistance(PointF p) const { return dot(normal(), p) - c; }
-	PointF project(PointF p) const { return p - signedDistance(p) * normal(); }
-
 	void reverse() { std::reverse(_points.begin(), _points.end()); }
 
-	void add(PointF p) {
-		assert(_directionInward != PointF());
-		_points.push_back(p);
-		if (_points.size() == 1)
-			c = dot(normal(), p);
-	}
-
-	void pop_back() { _points.pop_back(); }
-
-	void setDirectionInward(PointF d) { _directionInward = normalized(d); }
-
-	bool evaluate(double maxSignedDist = -1)
-	{
-		bool ret = evaluate(_points);
-		if (maxSignedDist > 0) {
-			auto points = _points;
-			while (true) {
-				auto old_points_size = points.size();
-				points.erase(
-					std::remove_if(points.begin(), points.end(),
-								   [this, maxSignedDist](auto p) { return this->signedDistance(p) > maxSignedDist; }),
-					points.end());
-				if (old_points_size == points.size())
-					break;
-#ifdef PRINT_DEBUG
-				printf("removed %zu points\n", old_points_size - points.size());
-#endif
-				ret = evaluate(points);
-			}
-		}
-		return ret;
-	}
-
-	double modules(PointF beg, PointF end) const
+	double modules(PointF beg, PointF end)
 	{
 		assert(_points.size() > 3);
+
+		// re-evaluate and filter out all points too far away. required for the gapSizes calculation.
+		evaluate(1.0, true);
+
 		std::vector<double> gapSizes;
 		gapSizes.reserve(_points.size());
 
@@ -518,11 +452,8 @@ public:
 		auto i = gapSizes.begin();
 		for (auto dist : gapSizes) {
 			sum += dist;
-			if (dist > 1.9 * unitPixelDist) {
-				// c++14: *i++ = std::exchange(sum, 0.0);
-				*i++ = sum;
-				sum = 0.0;
-			}
+			if (dist > 1.9 * unitPixelDist)
+				*i++ = std::exchange(sum, 0.0);
 		}
 		*i++ = sum + distance(end, project(_points.back()));
 		gapSizes.erase(i, gapSizes.end());
@@ -540,16 +471,6 @@ public:
 	}
 };
 
-PointF intersect(const RegressionLine& l1, const RegressionLine& l2)
-{
-	assert(l1.isValid() && l2.isValid());
-	double x, y, d;
-	d = l1.a * l2.b - l1.b * l2.a;
-	x = (l1.c * l2.b - l1.b * l2.c) / d;
-	y = (l1.a * l2.c - l1.c * l2.a) / d;
-	return {x, y};
-}
-
 class EdgeTracer : public BitMatrixCursorF
 {
 	enum class StepResult { FOUND, OPEN_END, CLOSED_END };
@@ -572,6 +493,13 @@ class EdgeTracer : public BitMatrixCursorF
 							// if we are not making any progress, we still have another endless loop bug
 							assert(p != centered(pEdge));
 							p = centered(pEdge);
+
+							if (history && maxStepSize == 1) {
+								if (history->get(PointI(p)) == state)
+									return StepResult::CLOSED_END;
+								history->set(PointI(p), state);
+							}
+
 							return StepResult::FOUND;
 						}
 						pEdge = pEdge - dEdge;
@@ -586,6 +514,9 @@ class EdgeTracer : public BitMatrixCursorF
 	}
 
 public:
+	ByteMatrix* history = nullptr;
+	int state = 0;
+
 	using BitMatrixCursorF::BitMatrixCursor;
 
 	bool updateDirectionFromOrigin(PointF origin)
@@ -597,9 +528,9 @@ public:
 			return false;
 		// make sure d stays in the same quadrant to prevent an infinite loop
 		if (std::abs(d.x) == std::abs(d.y))
-			d = mainDirection(old_d) + 0.99 * (d - mainDirection(old_d));
+			d = mainDirection(old_d) + 0.99f * (d - mainDirection(old_d));
 		else if (mainDirection(d) != mainDirection(old_d))
-			d = mainDirection(old_d) + 0.99 * mainDirection(d);
+			d = mainDirection(old_d) + 0.99f * mainDirection(d);
 		return true;
 	}
 
@@ -609,7 +540,7 @@ public:
 		do {
 			log(p);
 			line.add(p);
-			if (line.points().size() % 30 == 10) {
+			if (line.points().size() % 50 == 10) {
 				if (!line.evaluate())
 					return false;
 				if (!updateDirectionFromOrigin(p - line.project(p) + line.points().front()))
@@ -621,12 +552,15 @@ public:
 		} while (true);
 	}
 
-	bool traceGaps(PointF dEdge, RegressionLine& line, int maxStepSize, const RegressionLine& finishLine)
+	bool traceGaps(PointF dEdge, RegressionLine& line, int maxStepSize, const RegressionLine& finishLine = {})
 	{
 		line.setDirectionInward(dEdge);
 		int gaps = 0;
 		do {
+			// detect an endless loop (lack of progress). if encountered, please report.
 			assert(line.points().empty() || p != line.points().back());
+			if (!line.points().empty() && p == line.points().back())
+				return false;
 			log(p);
 
 			// if we drifted too far outside of the code, break
@@ -646,7 +580,10 @@ public:
 				// make sure we are making progress even when back-projecting:
 				// consider a 90deg corner, rotated 45deg. we step away perpendicular from the line and get
 				// back projected where we left off the line.
-				if (distance(np, line.project(line.points().back())) < 1)
+				// The 'while' instead of 'if' was introduced to fix the issue with #245. It turns out that
+				// np can actually be behind the projection of the last line point and we need 2 steps in d
+				// to prevent a dead lock. see #245.png
+				while (distance(np, line.project(line.points().back())) < 1)
 					np = np + d;
 				p = centered(np);
 			}
@@ -702,178 +639,197 @@ public:
 	}
 };
 
-
-static DetectorResult SampleGrid(const BitMatrix& image, PointF tl, PointF bl, PointF br, PointF tr, int width, int height)
+static DetectorResult Scan(EdgeTracer startTracer, std::array<DMRegressionLine, 4>& lines)
 {
-	auto moveTowardsBy = [](PointF& a, PointF b, double d) { a = movedTowardsBy(a, b, d); };
+	while (startTracer.step()) {
+		log(startTracer.p);
 
-	// shrink shape by half a pixel to go from center of white pixel outside of code to the edge between white and black
-	moveTowardsBy(tl, br, 0.5);
-	moveTowardsBy(br, tl, 0.5);
-	moveTowardsBy(bl, tr, 0.5);
-	// move the tr point a little less because the jagged top and right line tend to be statistically slightly
-	// inclined toward the center anyway.
-	moveTowardsBy(tr, bl, 0.3);
+		// continue until we cross from black into white
+		if (!startTracer.edgeAtBack().isWhite())
+			continue;
 
-	return SampleGrid(image, width, height, PerspectiveTransform(Rectangle(width, height, 0), {tl, tr, br, bl}));
+		PointF tl, bl, br, tr;
+		auto& [lineL, lineB, lineR, lineT] = lines;
+
+		for (auto& l : lines)
+			l.reset();
+
+#ifdef PRINT_DEBUG
+		SCOPE_EXIT([&] {
+			for (auto& l : lines)
+				log(l.points());
+		});
+# define CHECK(A) if (!(A)) { printf("broke at %d\n", __LINE__); continue; }
+#else
+# define CHECK(A) if(!(A)) continue
+#endif
+
+		auto t = startTracer;
+
+		// follow left leg upwards
+		t.turnRight();
+		t.state = 1;
+		CHECK(t.traceLine(t.right(), lineL));
+		CHECK(t.traceCorner(t.right(), tl));
+		lineL.reverse();
+		auto tlTracer = t;
+
+		// follow left leg downwards
+		t = startTracer;
+		t.state = 1;
+		t.setDirection(tlTracer.right());
+		CHECK(t.traceLine(t.left(), lineL));
+		if (!lineL.isValid())
+			t.updateDirectionFromOrigin(tl);
+		auto up = t.back();
+		CHECK(t.traceCorner(t.left(), bl));
+
+		// follow bottom leg right
+		t.state = 2;
+		CHECK(t.traceLine(t.left(), lineB));
+		if (!lineB.isValid())
+			t.updateDirectionFromOrigin(bl);
+		auto right = t.front();
+		CHECK(t.traceCorner(t.left(), br));
+
+		auto lenL = distance(tl, bl) - 1;
+		auto lenB = distance(bl, br) - 1;
+		CHECK(lenL >= 8 && lenB >= 10 && lenB >= lenL / 4 && lenB <= lenL * 18);
+
+		auto maxStepSize = static_cast<int>(lenB / 5 + 1); // datamatrix bottom dim is at least 10
+
+		// at this point we found a plausible L-shape and are now looking for the b/w pattern at the top and right:
+		// follow top row right 'half way' (4 gaps), see traceGaps break condition with 'invalid' line
+		tlTracer.setDirection(right);
+		CHECK(tlTracer.traceGaps(tlTracer.right(), lineT, maxStepSize));
+
+		maxStepSize = std::min(lineT.length() / 3, static_cast<int>(lenL / 5)) * 2;
+
+		// follow up until we reach the top line
+		t.setDirection(up);
+		t.state = 3;
+		CHECK(t.traceGaps(t.left(), lineR, maxStepSize, lineT));
+		CHECK(t.traceCorner(t.left(), tr));
+
+		auto lenT = distance(tl, tr) - 1;
+		auto lenR = distance(tr, br) - 1;
+
+		CHECK(std::abs(lenT - lenB) / lenB < 0.5 && std::abs(lenR - lenL) / lenL < 0.5 &&
+			  lineT.points().size() >= 5 && lineR.points().size() >= 5);
+
+		// continue top row right until we cross the right line
+		CHECK(tlTracer.traceGaps(tlTracer.right(), lineT, maxStepSize, lineR));
+
+#ifdef PRINT_DEBUG
+		printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f (%d : %d : %d : %d)\n", bl.x, bl.y,
+			   tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, (int)lenL, (int)lenB, (int)lenT, (int)lenR);
+#endif
+
+		for (auto* l : {&lineL, &lineB, &lineT, &lineR})
+			l->evaluate(1.0);
+
+		// find the bounding box corners of the code with sub-pixel precision by intersecting the 4 border lines
+		bl = intersect(lineB, lineL);
+		tl = intersect(lineT, lineL);
+		tr = intersect(lineT, lineR);
+		br = intersect(lineB, lineR);
+
+		int dimT, dimR;
+		double fracT, fracR;
+		auto splitDouble = [](double d, int* i, double* f) {
+			*i = std::isnormal(d) ? static_cast<int>(d + 0.5) : 0;
+			*f = std::isnormal(d) ? std::abs(d - *i) : INFINITY;
+		};
+		splitDouble(lineT.modules(tl, tr), &dimT, &fracT);
+		splitDouble(lineR.modules(br, tr), &dimR, &fracR);
+
+#ifdef PRINT_DEBUG
+		printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f ^> %.1f, %.1f\n", bl.x, bl.y,
+			   tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, tr.x, tr.y);
+		printf("dim: %d x %d\n", dimT, dimR);
+#endif
+
+		// if we have an almost square (invalid rectangular) data matrix dimension, we try to parse it by assuming a
+		// square. we use the dimension that is closer to an integral value. all valid rectangular symbols differ in
+		// their dimension by at least 10 (here 5, see doubling below). Note: this is currently not required for the
+		// black-box tests to complete.
+		if (std::abs(dimT - dimR) < 5)
+			dimT = dimR = fracR < fracT ? dimR : dimT;
+
+		// the dimension is 2x the number of black/white transitions
+		dimT *= 2;
+		dimR *= 2;
+
+		CHECK(dimT >= 10 && dimT <= 144 && dimR >= 8 && dimR <= 144);
+
+		auto movedTowardsBy = [](PointF& a, PointF b1, PointF b2, auto d) {
+			return a + d * normalized(normalized(b1 - a) + normalized(b2 - a));
+		};
+
+		// shrink shape by half a pixel to go from center of white pixel outside of code to the edge between white and black
+		QuadrilateralF sourcePoints = {
+			movedTowardsBy(tl, tr, bl, 0.5f),
+			// move the tr point a little less because the jagged top and right line tend to be statistically slightly
+			// inclined toward the center anyway.
+			movedTowardsBy(tr, br, tl, 0.3f),
+			movedTowardsBy(br, bl, tr, 0.5f),
+			movedTowardsBy(bl, tl, br, 0.5f),
+		};
+
+		auto res = SampleGrid(*startTracer.img, dimT, dimR, PerspectiveTransform(Rectangle(dimT, dimR, 0), sourcePoints));
+
+		CHECK(res.isValid());
+
+		return res;
+	}
+
+	return {};
 }
 
-static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
+static DetectorResult DetectNew(const BitMatrix& image, bool tryHarder, bool tryRotate)
 {
-	// walk to the left at first
 #ifdef PRINT_DEBUG
 	LogMatrixWriter lmw(log, image, 1, "dm-log.pnm");
-	for (auto startDirection : {PointF(-1, 0)}) {
-#else
-	for (auto startDirection : {PointF(-1, 0), PointF(-1, 0), PointF(1, 0), PointF(0, -1), PointF(0, 1)}) {
+//	tryRotate = tryHarder = false;
 #endif
-		// TODO: If neither the horizontal nor the vertical center line cross the symbol, it will be overlooked.
-		// Will need to look at more lines.
-		auto center = PointF(image.width()/2, image.height()/2);
-		EdgeTracer startTracer(image, centered(center - center * startDirection), startDirection);
-		while (startTracer.step()) {
-			log(startTracer.p);
 
-			// continue until we cross from black into white
-			if (!startTracer.edgeAtBack().isWhite())
-				continue;
+	// disable expensive multi-line scan to detect off-center symbols for now
+	tryHarder = false;
 
-#ifdef PRINT_DEBUG
-#define continue { \
-			printf("broke at %d\n", __LINE__); \
-			for (auto* l : {&lineL, &lineB, &lineT, &lineR}) log(l->points()); \
-			continue; \
+	// a history log to remember where the tracing already passed by to prevent a later trace from doing the same work twice
+	ByteMatrix history;
+	if (tryHarder)
+		history = ByteMatrix(image.width(), image.height());
+
+	// instantiate RegressionLine objects outside of Scan function to prevent repetitive std::vector allocations
+	std::array<DMRegressionLine, 4> lines;
+
+	constexpr int minSymbolSize = 8 * 2; // minimum realistic size in pixel: 8 modules x 2 pixels per module
+
+	for (auto dir : {PointF(-1, 0), PointF(1, 0), PointF(0, -1), PointF(0, 1)}) {
+		auto center = PointF(image.width() / 2, image.height() / 2);
+		auto startPos = centered(center - center * dir + minSymbolSize / 2 * dir);
+
+		EdgeTracer tracer(image, startPos, dir);
+		if (tryHarder) {
+			tracer.history = &history;
+			history.clear();
 		}
-#endif
 
-			PointF tl, bl, br, tr;
-			RegressionLine lineL, lineB, lineR, lineT;
+		for (int i = 1;; ++i) {
+			tracer.p = startPos + i / 2 * minSymbolSize * (i & 1 ? -1 : 1) * tracer.right();
 
-			auto t = startTracer;
+			if (!tracer.isIn())
+				break;
 
-			// follow left leg upwards
-			t.turnRight();
-			if (!t.traceLine(t.right(), lineL))
-				continue;
+			if (auto res = Scan(tracer, lines); res.isValid())
+				return res;
 
-			if (!t.traceCorner(t.right(), tl))
-				continue;
-			lineL.reverse();
-			auto tlTracer = t;
-
-			// follow left leg downwards
-			t = startTracer;
-			t.setDirection(tlTracer.right());
-			if (!t.traceLine(t.left(), lineL))
-				continue;
-
-			if (!lineL.isValid())
-				t.updateDirectionFromOrigin(tl);
-			auto up = t.back();
-			if (!t.traceCorner(t.left(), bl))
-				continue;
-
-			// follow bottom leg right
-			if (!t.traceLine(t.left(), lineB))
-				continue;
-
-			if (!lineB.isValid())
-				t.updateDirectionFromOrigin(bl);
-			auto right = t.front();
-			if (!t.traceCorner(t.left(), br))
-				continue;
-
-			auto lenL = distance(tl, bl) - 1;
-			auto lenB = distance(bl, br) - 1;
-			if (lenL < 10 || lenB < 10 || lenB < lenL / 4 || lenB > lenL * 8)
-				continue;
-
-			auto maxStepSize = static_cast<int>(lenB / 5 + 1); // datamatrix dim is at least 10x10
-
-			// at this point we found a plausible L-shape and are now looking for the b/w pattern at the top and right:
-			// follow top row right 'half way' (4 gaps), see traceGaps break condition with 'invalid' line
-			tlTracer.setDirection(right);
-			if (!tlTracer.traceGaps(tlTracer.right(), lineT, maxStepSize, RegressionLine()))
-				continue;
-
-			maxStepSize = std::min(lineT.length() / 3, static_cast<int>(lenL / 5)) * 2;
-
-			// follow up until we reach the top line
-			t.setDirection(up);
-			if (!t.traceGaps(t.left(), lineR, maxStepSize, lineT))
-				continue;
-
-			if (!t.traceCorner(t.left(), tr))
-				continue;
-
-			auto lenT = distance(tl, tr) - 1;
-			auto lenR = distance(tr, br) - 1;
-
-			if (std::abs(lenT - lenB) / lenB > 0.5 || std::abs(lenR - lenL) / lenL > 0.5 ||
-			        lineT.points().size() < 5 || lineR.points().size() < 5)
-				continue;
-
-			// continue top row right until we cross the right line
-			if (!tlTracer.traceGaps(tlTracer.right(), lineT, maxStepSize, lineR))
-				continue;
-
-#ifdef PRINT_DEBUG
-			printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f (%d : %d : %d : %d)\n", bl.x, bl.y,
-			       tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, (int)lenL, (int)lenB, (int)lenT, (int)lenR);
-#endif
-
-			for (RegressionLine* l : {&lineL, &lineB, &lineT, &lineR})
-				l->evaluate(1.0);
-
-			// find the bounding box corners of the code with sub-pixel precision by intersecting the 4 border lines
-			bl = intersect(lineB, lineL);
-			tl = intersect(lineT, lineL);
-			tr = intersect(lineT, lineR);
-			br = intersect(lineB, lineR);
-
-			int dimT, dimR;
-			double fracT, fracR;
-			auto splitDouble = [](double d, int* i, double* f) {
-				*i = std::isnormal(d) ? static_cast<int>(d + 0.5) : 0;
-				*f = std::isnormal(d) ? std::abs(d - *i) : INFINITY;
-			};
-			splitDouble(lineT.modules(tl, tr), &dimT, &fracT);
-			splitDouble(lineR.modules(br, tr), &dimR, &fracR);
-
-#ifdef PRINT_DEBUG
-			printf("L: %.1f, %.1f ^ %.1f, %.1f > %.1f, %.1f ^> %.1f, %.1f\n", bl.x, bl.y,
-			       tl.x - bl.x, tl.y - bl.y, br.x - bl.x, br.y - bl.y, tr.x, tr.y);
-			printf("dim: %d x %d\n", dimT, dimR);
-#endif
-
-			// if we have an invalid rectangular data matrix dimension, we try to parse it by assuming a square
-			// we use the dimension that is closer to an integral value
-			if (dimT < 2 * dimR || dimT > 4 * dimR)
-				dimT = dimR = fracR < fracT ? dimR : dimT;
-
-			// the dimension is 2x the number of black/white transitions
-			dimT *= 2;
-			dimR *= 2;
-
-			if (dimT < 10 || dimT > 144 || dimR < 8 || dimR > 144 )
-				continue;
-
-			auto res = SampleGrid(image, tl, bl, br, tr, dimT, dimR);
-
-#ifdef PRINT_DEBUG
-			for (RegressionLine* l : {&lineL, &lineB, &lineT, &lineR})
-				log(l->points());
-#endif
-
-			if (!res.isValid())
-				continue;
-
-			return res;
+			if (!tryHarder)
+				break; // only test center lines
 		}
-		// reached border of image -> try next scan direction
-#ifndef PRINT_DEBUG
+
 		if (!tryRotate)
-#endif
 			break; // only test left direction
 	}
 
@@ -888,45 +844,47 @@ static DetectorResult DetectNew(const BitMatrix& image, bool tryRotate)
 */
 static DetectorResult DetectPure(const BitMatrix& image)
 {
-	const int minSize = 8; // datamatrix codes are at least 8x8 modules
 	int left, top, width, height;
-	if (!image.findBoundingBox(left, top, width, height, minSize)) {
+	if (!image.findBoundingBox(left, top, width, height, 8))
 		return {};
-	}
 
-	// find the first white pixel on the diagonal
-	int moduleSize = 1;
-	while (moduleSize < width / minSize && image.get(left + moduleSize, top))
-		++moduleSize;
+	BitMatrixCursorI cur(image, {left, top}, {0, 1});
+	if (cur.countEdges(height - 1) != 0)
+		return {};
+	cur.turnLeft();
+	if (cur.countEdges(width - 1) != 0)
+		return {};
+	cur.turnLeft();
+	int dimR = cur.countEdges(height - 1) + 1;
+	cur.turnLeft();
+	int dimT = cur.countEdges(width - 1) + 1;
 
-	int matrixWidth = width / moduleSize;
-	int matrixHeight = height / moduleSize;
-	if (matrixWidth < minSize || matrixHeight < minSize) {
+	auto modSizeX = float(width) / dimT;
+	auto modSizeY = float(height) / dimR;
+	auto modSize = (modSizeX + modSizeY) / 2;
+
+	if (dimT % 2 != 0 || dimR % 2 != 0 || dimT < 10 || dimT > 144 || dimR < 8 || dimR > 144 ||
+		std::abs(modSizeX - modSizeY) > 1 ||
+		!image.isIn(PointF{left + modSizeX / 2 + (dimT - 1) * modSize, top + modSizeY / 2 + (dimR - 1) * modSize}))
 		return {};
-	}
 
-	// Push in the "border" by half the module width so that we start
-	// sampling in the middle of the module. Just in case the image is a
-	// little off, this will help recover.
-	int msh    = moduleSize / 2;
 	int right  = left + width - 1;
 	int bottom = top + height - 1;
 
 	// Now just read off the bits (this is a crop + subsample)
-	return {Deflate(image, matrixWidth, matrixHeight, top + msh, left + msh, moduleSize),
+	return {Deflate(image, dimT, dimR, top + modSizeX / 2, left + modSizeY / 2, modSize),
 			{{left, top}, {right, top}, {right, bottom}, {left, bottom}}};
 }
 
-DetectorResult Detector::Detect(const BitMatrix& image, bool tryHarder, bool tryRotate, bool isPure)
+DetectorResult Detect(const BitMatrix& image, bool tryHarder, bool tryRotate, bool isPure)
 {
 	if (isPure)
 		return DetectPure(image);
 
-	auto result = DetectNew(image, tryRotate);
+	auto result = DetectNew(image, tryHarder, tryRotate);
 	if (!result.isValid() && tryHarder)
 		result = DetectOld(image);
 	return result;
 }
 
-} // DataMatrix
-} // ZXing
+} // namespace ZXing::DataMatrix
