@@ -16,26 +16,25 @@
 */
 
 #include "DMHighLevelEncoder.h"
+
+#include "ByteArray.h"
+#include "CharacterSet.h"
 #include "DMEncoderContext.h"
 #include "TextEncoder.h"
-#include "CharacterSet.h"
-#include "ByteArray.h"
-#include "ZXStrConvWorkaround.h"
 #include "ZXContainerAlgorithms.h"
 
-#include <cstddef>
-#include <cstdint>
-#include <string>
 #include <algorithm>
 #include <array>
-#include <limits>
 #include <cmath>
-#include <numeric>
+#include <cstddef>
+#include <cstdint>
 #include <functional>
+#include <limits>
+#include <numeric>
 #include <stdexcept>
+#include <string>
 
-namespace ZXing {
-namespace DataMatrix {
+namespace ZXing::DataMatrix {
 
 static const uint8_t PAD = 129;
 static const uint8_t UPPER_SHIFT = 235;
@@ -67,44 +66,44 @@ static const uint8_t LATCHES[] = {
 	231, // LATCH_TO_BASE256,
 };
 
-static inline bool IsDigit(int ch)
+static bool IsDigit(int ch)
 {
 	return ch >= '0' && ch <= '9';
 }
 
-static inline bool IsExtendedASCII(int ch)
+static bool IsExtendedASCII(int ch)
 {
 	return ch >= 128 && ch <= 255;
 }
 
-static inline bool IsNativeC40(int ch)
+static bool IsNativeC40(int ch)
 {
 	return (ch == ' ') || (ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z');
 }
 
-static inline bool IsNativeText(int ch)
+static bool IsNativeText(int ch)
 {
 	return (ch == ' ') || (ch >= '0' && ch <= '9') || (ch >= 'a' && ch <= 'z');
 }
 
-static inline bool IsX12TermSep(int ch)
+static bool IsX12TermSep(int ch)
 {
 	return (ch == '\r') //CR
 		|| (ch == '*')
 		|| (ch == '>');
 }
 
-static inline bool IsNativeX12(int ch)
+static bool IsNativeX12(int ch)
 {
 	return IsX12TermSep(ch) || (ch == ' ') || (ch >= '0' && ch <= '9') || (ch >= 'A' && ch <= 'Z');
 }
 
-static inline bool IsNativeEDIFACT(int ch)
+static bool IsNativeEDIFACT(int ch)
 {
 	return ch >= ' ' && ch <= '^';
 }
 
-static inline bool IsSpecialB256(int /*ch*/)
+static bool IsSpecialB256(int /*ch*/)
 {
 	return false; //TODO NOT IMPLEMENTED YET!!!
 }
@@ -608,26 +607,16 @@ namespace X12Encoder {
 	static int EncodeChar(int c, std::string& sb)
 	{
 		switch (c) {
-		case '\r':
-			sb.push_back('\0');
-			break;
-		case '*':
-			sb.push_back('\1');
-			break;
-		case '>':
-			sb.push_back('\2');
-			break;
-		case ' ':
-			sb.push_back('\3');
-			break;
+		case '\r': sb.push_back('\0'); break;
+		case '*': sb.push_back('\1'); break;
+		case '>': sb.push_back('\2'); break;
+		case ' ': sb.push_back('\3'); break;
 		default:
 			if (c >= '0' && c <= '9') {
 				sb.push_back((char)(c - 48 + 4));
-			}
-			else if (c >= 'A' && c <= 'Z') {
+			} else if (c >= 'A' && c <= 'Z') {
 				sb.push_back((char)(c - 65 + 14));
-			}
-			else {
+			} else {
 				throw std::invalid_argument("Illegal character: " + ToHexString(c));
 			}
 			break;
@@ -874,8 +863,7 @@ static bool EndsWith(const std::wstring& s, const std::wstring& ss)
 	return s.length() > ss.length() && s.compare(s.length() - ss.length(), ss.length(), ss) == 0;
 }
 
-ByteArray
-HighLevelEncoder::Encode(const std::wstring& msg)
+ByteArray Encode(const std::wstring& msg)
 {
 	return Encode(msg, SymbolShape::NONE, -1, -1, -1, -1);
 }
@@ -891,8 +879,7 @@ HighLevelEncoder::Encode(const std::wstring& msg)
 * @param maxSize the maximum symbol size constraint or null for no constraint
 * @return the encoded message (the char values range from 0 to 255)
 */
-ByteArray
-HighLevelEncoder::Encode(const std::wstring& msg, SymbolShape shape, int minWdith, int minHeight, int maxWidth, int maxHeight)
+ByteArray Encode(const std::wstring& msg, SymbolShape shape, int minWidth, int minHeight, int maxWidth, int maxHeight)
 {
 	//the codewords 0..255 are encoded as Unicode characters
 	//Encoder[] encoders = {
@@ -902,7 +889,7 @@ HighLevelEncoder::Encode(const std::wstring& msg, SymbolShape shape, int minWdit
 
 	EncoderContext context(TextEncoder::FromUnicode(msg, CharacterSet::ISO8859_1));
 	context.setSymbolShape(shape);
-	context.setSizeConstraints(minWdith, minHeight, maxWidth, maxHeight);
+	context.setSizeConstraints(minWidth, minHeight, maxWidth, maxHeight);
 
 	if (StartsWith(msg, MACRO_05_HEADER) && EndsWith(msg, MACRO_TRAILER)) {
 		context.addCodeword(MACRO_05);
@@ -918,12 +905,12 @@ HighLevelEncoder::Encode(const std::wstring& msg, SymbolShape shape, int minWdit
 	int encodingMode = ASCII_ENCODATION; //Default mode
 	while (context.hasMoreCharacters()) {
 		switch (encodingMode) {
-			case ASCII_ENCODATION:   ASCIIEncoder::EncodeASCII(context);     break;
-			case C40_ENCODATION:     C40Encoder::EncodeC40(context);         break;
-			case TEXT_ENCODATION:    DMTextEncoder::EncodeText(context);     break;
-			case X12_ENCODATION:     X12Encoder::EncodeX12(context);         break;
-			case EDIFACT_ENCODATION: EdifactEncoder::EncodeEdifact(context); break;
-			case BASE256_ENCODATION: Base256Encoder::EncodeBase256(context); break;
+		case ASCII_ENCODATION:   ASCIIEncoder::EncodeASCII(context);     break;
+		case C40_ENCODATION:     C40Encoder::EncodeC40(context);         break;
+		case TEXT_ENCODATION:    DMTextEncoder::EncodeText(context);     break;
+		case X12_ENCODATION:     X12Encoder::EncodeX12(context);         break;
+		case EDIFACT_ENCODATION: EdifactEncoder::EncodeEdifact(context); break;
+		case BASE256_ENCODATION: Base256Encoder::EncodeBase256(context); break;
 		}
 		if (context.newEncoding() >= 0) {
 			encodingMode = context.newEncoding();
@@ -949,5 +936,4 @@ HighLevelEncoder::Encode(const std::wstring& msg, SymbolShape shape, int minWdit
 	return context.codewords();
 }
 
-} // DataMatrix
-} // ZXing
+} // namespace ZXing::DataMatrix
