@@ -24,9 +24,6 @@
 #include "GTIN.h"
 #include "ODUPCEANCommon.h"
 #include "Result.h"
-#include "TextDecoder.h"
-
-#include <cmath>
 
 namespace ZXing::OneD {
 
@@ -51,16 +48,17 @@ static const int FIRST_DIGIT_ENCODINGS[] = {
 	0x00, 0x0B, 0x0D, 0x0E, 0x13, 0x19, 0x1C, 0x15, 0x16, 0x1A
 };
 
-// The GS1 specification has the following to say about quite zones
+// The GS1 specification has the following to say about quiet zones
 // Type: EAN-13 | EAN-8 | UPC-A | UPC-E | EAN Add-on | UPC Add-on
 // QZ L:   11   |   7   |   9   |   9   |     7-12   |     9-12
 // QZ R:    7   |   7   |   9   |   7   |        5   |        5
 
 constexpr float QUIET_ZONE_LEFT = 6;
 constexpr float QUIET_ZONE_RIGHT = 6;
+constexpr float QUIET_ZONE_ADDON = 3;
 
 // There is a single sample (ean13-1/12.png) that fails to decode with these (new) settings because
-// it has a right-side quite zone of only about 4.5 modules, which is clearly out of spec.
+// it has a right-side quiet zone of only about 4.5 modules, which is clearly out of spec.
 
 static bool DecodeDigit(const PatternView& view, std::string& txt, int* lgPattern = nullptr)
 {
@@ -254,6 +252,8 @@ static bool AddOn(PartialResult& res, PatternView begin, int digitCount)
 	auto moduleSize = IsPattern(ext, EXT_START_PATTERN);
 	CHECK(moduleSize);
 
+	CHECK(ext.isAtLastBar() || *ext.end() > QUIET_ZONE_ADDON * moduleSize - 1);
+
 	res.end = ext;
 	ext = ext.subView(EXT_START_PATTERN.size(), CHAR_LEN);
 	int lgPattern = 0;
@@ -268,8 +268,6 @@ static bool AddOn(PartialResult& res, PatternView begin, int digitCount)
 		}
 	}
 
-	//TODO: check right quite zone
-
 	if (digitCount == 2) {
 		CHECK(std::stoi(res.txt) % 4 == lgPattern);
 	} else {
@@ -280,15 +278,16 @@ static bool AddOn(PartialResult& res, PatternView begin, int digitCount)
 	return true;
 }
 
-Result MultiUPCEANReader::decodePattern(int rowNumber, const PatternView& row, std::unique_ptr<RowReader::DecodingState>&) const
+Result MultiUPCEANReader::decodePattern(int rowNumber, PatternView& next, std::unique_ptr<RowReader::DecodingState>&) const
 {
 	const int minSize = 3 + 6*4 + 6; // UPC-E
 
-	auto begin = FindLeftGuard(row, minSize, END_PATTERN, QUIET_ZONE_LEFT);
-	if (!begin.isValid())
+	next = FindLeftGuard(next, minSize, END_PATTERN, QUIET_ZONE_LEFT);
+	if (!next.isValid())
 		return Result(DecodeStatus::NotFound);
 
 	PartialResult res;
+	auto begin = next;
 
 	if (!(((_hints.hasFormat(BarcodeFormat::EAN13 | BarcodeFormat::UPCA)) && EAN13(res, begin)) ||
 		  (_hints.hasFormat(BarcodeFormat::EAN8) && EAN8(res, begin)) ||
@@ -306,19 +305,32 @@ Result MultiUPCEANReader::decodePattern(int rowNumber, const PatternView& row, s
 		res.format = BarcodeFormat::UPCA;
 	}
 
+	// Symbology identifier modifiers ISO/IEC 15420:2009 Annex B Table B.1
+	// ISO/IEC 15420:2009 (& GS1 General Specifications 5.1.3) states that the content for "]E0" should be 13 digits,
+	// i.e. converted to EAN-13 if UPC-A/E, but not doing this here to maintain backward compatibility
+	std::string symbologyIdentifier(res.format == BarcodeFormat::EAN8 ? "]E4" : "]E0");
+
 	auto ext = res.end;
 	PartialResult addOnRes;
 	if (_hints.eanAddOnSymbol() != EanAddOnSymbol::Ignore && ext.skipSymbol() &&
 		ext.skipSingle(static_cast<int>(begin.sum() * 3.5)) && (AddOn(addOnRes, ext, 5) || AddOn(addOnRes, ext, 2))) {
 
+		// ISO/IEC 15420:2009 states that the content for "]E3" should be 15 or 18 digits, i.e. converted to EAN-13
+		// and extended with no separator, and that the content for "]E4" should be 8 digits, i.e. no add-on
 		//TODO: extend position in include extension
 		res.txt += " " + addOnRes.txt;
+
+		if (res.format != BarcodeFormat::EAN8) // Keeping EAN-8 with add-on as "]E4"
+			symbologyIdentifier = "]E3"; // Combined packet, EAN-13, UPC-A, UPC-E, with add-on
 	}
 
+	next = res.end;
+
 	if (_hints.eanAddOnSymbol() == EanAddOnSymbol::Require && !addOnRes.isValid())
 		return Result(DecodeStatus::NotFound);
 
-	return {res.txt, rowNumber, begin.pixelsInFront(), res.end.pixelsTillEnd(), res.format};
+	return {
+		res.txt, rowNumber, begin.pixelsInFront(), res.end.pixelsTillEnd(), res.format, std::move(symbologyIdentifier)};
 }
 
 } // namespace ZXing::OneD
