@@ -22,6 +22,8 @@
 
 #include <algorithm>
 #include <cstdlib>
+#include <cstring>
+#include <iterator>
 
 namespace ZXing::OneD::DataBar {
 
@@ -33,6 +35,7 @@ struct AiInfo
 	int fieldSize;	// if negative, the length is variable and abs(length) give the max size
 };
 
+// GS1 General Specifications Release 22.0 (Jan 22, 2022)
 static const AiInfo aiInfos[] = {
 // TWO_DIGIT_DATA_LENGTH
 	{ "00", 18 },
@@ -44,35 +47,39 @@ static const AiInfo aiInfos[] = {
 	{ "12", 6 },
 	{ "13", 6 },
 	{ "15", 6 },
+	{ "16", 6 },
 	{ "17", 6 },
 
 	{ "20", 2 },
 	{ "21", -20 },
-	{ "22", -29 },
+	{ "22", -20 },
 
 	{ "30", -8 },
 	{ "37", -8 },
 
 	//internal company codes
 	{ "90", -30 },
-	{ "91", -30 },
-	{ "92", -30 },
-	{ "93", -30 },
-	{ "94", -30 },
-	{ "95", -30 },
-	{ "96", -30 },
-	{ "97", -30 },
-	{ "98", -30 },
-	{ "99", -30 },
+	{ "91", -90 },
+	{ "92", -90 },
+	{ "93", -90 },
+	{ "94", -90 },
+	{ "95", -90 },
+	{ "96", -90 },
+	{ "97", -90 },
+	{ "98", -90 },
+	{ "99", -90 },
 
 //THREE_DIGIT_DATA_LENGTH
+	{ "235", -28 },
 	{ "240", -30 },
 	{ "241", -30 },
 	{ "242", -6 },
+	{ "243", -20 },
 	{ "250", -30 },
 	{ "251", -30 },
-	{ "253", -17 },
+	{ "253", -30 },
 	{ "254", -20 },
+	{ "255", -25 },
 
 	{ "400", -30 },
 	{ "401", -30 },
@@ -83,13 +90,24 @@ static const AiInfo aiInfos[] = {
 	{ "412", 13 },
 	{ "413", 13 },
 	{ "414", 13 },
+	{ "415", 13 },
+	{ "416", 13 },
+	{ "417", 13 },
 	{ "420", -20 },
-	{ "421", -15 },
+	{ "421", -12 },
 	{ "422", 3 },
 	{ "423", -15 },
 	{ "424", 3 },
-	{ "425", 3 },
+	{ "425", -15 },
 	{ "426", 3 },
+	{ "427", -3 },
+
+	{ "710", -20 },
+	{ "711", -20 },
+	{ "712", -20 },
+	{ "713", -20 },
+	{ "714", -20 },
+	{ "715", -20 },
 
 //THREE_DIGIT_PLUS_DIGIT_DATA_LENGTH
 	{ "310", 6 },
@@ -116,6 +134,7 @@ static const AiInfo aiInfos[] = {
 	{ "334", 6 },
 	{ "335", 6 },
 	{ "336", 6 },
+	{ "337", 6 },
 	{ "340", 6 },
 	{ "341", 6 },
 	{ "342", 6 },
@@ -148,12 +167,55 @@ static const AiInfo aiInfos[] = {
 	{ "391", -18 },
 	{ "392", -15 },
 	{ "393", -18 },
+	{ "394", 4 },
+	{ "395", 6 },
 	{ "703", -30 },
+	{ "723", -30 },
 
 //FOUR_DIGIT_DATA_LENGTH
+	{ "4300", -35 },
+	{ "4301", -35 },
+	{ "4302", -70 },
+	{ "4303", -70 },
+	{ "4304", -70 },
+	{ "4305", -70 },
+	{ "4306", -70 },
+	{ "4307", 2 },
+	{ "4308", -30 },
+	{ "4310", -35 },
+	{ "4311", -35 },
+	{ "4312", -70 },
+	{ "4313", -70 },
+	{ "4314", -70 },
+	{ "4315", -70 },
+	{ "4316", -70 },
+	{ "4317", 2 },
+	{ "4318", -20 },
+	{ "4319", -30 },
+	{ "4320", -35 },
+	{ "4321", 1 },
+	{ "4322", 1 },
+	{ "4323", 1 },
+	{ "4324", 10 },
+	{ "4325", 10 },
+	{ "4326", 6 },
+
 	{ "7001", 13 },
 	{ "7002", -30 },
 	{ "7003", 10 },
+	{ "7004", -4 },
+	{ "7005", -12 },
+	{ "7006", 6 },
+	{ "7007", -12 },
+	{ "7008", -3 },
+	{ "7009", -10 },
+	{ "7010", -2 },
+	{ "7020", -20 },
+	{ "7021", -20 },
+	{ "7022", -20 },
+	{ "7023", -30 },
+	{ "7040", 4 },
+	{ "7240", -20 },
 
 	{ "8001", 14 },
 	{ "8002", -20 },
@@ -161,21 +223,29 @@ static const AiInfo aiInfos[] = {
 	{ "8004", -30 },
 	{ "8005", 6 },
 	{ "8006", 18 },
-	{ "8007", -30 },
+	{ "8007", -34 },
 	{ "8008", -12 },
+	{ "8009", -50 },
+	{ "8010", -30 },
+	{ "8011", -12 },
+	{ "8012", -20 },
+	{ "8013", -25 },
+	{ "8017", 18 },
 	{ "8018", 18 },
+	{ "8019", -10 },
 	{ "8020", -25 },
-	{ "8100", 6 },
-	{ "8101", 10 },
-	{ "8102", 2 },
+	{ "8026", 18 },
 	{ "8110", -70 },
+	{ "8111", 4 },
+	{ "8112", -70 },
 	{ "8200", -70 },
 };
 
 static size_t
 AiSize(const char* aiPrefix)
 {
-	if ((aiPrefix[0] == '3' && Contains("1234569", aiPrefix[1])) || std::string(aiPrefix) == "703")
+	if ((aiPrefix[0] == '3' && Contains("1234569", aiPrefix[1])) || std::string(aiPrefix) == "703"
+			|| std::string(aiPrefix) == "723")
 		return 4;
 	else
 		return strlen(aiPrefix);
@@ -215,4 +285,4 @@ ParseFieldsInGeneralPurpose(const std::string &rawInfo, std::string& result)
 	return status;
 }
 
-} // namespace ZXing::OneD::RSS
+} // namespace ZXing::OneD::DataBar
