@@ -16,29 +16,16 @@
 */
 
 #include "ODRSSGenericAppIdDecoder.h"
-#include "ODRSSFieldParser.h"
+
 #include "BitArray.h"
 #include "DecodeStatus.h"
-#include "ZXStrConvWorkaround.h"
+#include "ODRSSFieldParser.h"
 
 #include <limits>
 #include <stdexcept>
 #include <utility>
 
-namespace ZXing {
-namespace OneD {
-namespace RSS {
-
-int
-GenericAppIdDecoder::ExtractNumeric(const BitArray& bits, int pos, int count)
-{
-	int value = 0;
-	auto bitIter = bits.iterAt(pos);
-	for (int i = 0; i < count; ++i, ++bitIter) {
-		value = (value << 1) | static_cast<int>(*bitIter);
-	}
-	return value;
-}
+namespace ZXing::OneD::DataBar {
 
 struct DecodedValue
 {
@@ -130,8 +117,6 @@ struct ParsingState
 	State encoding = NUMERIC;
 };
 
-#define ExtractNumeric GenericAppIdDecoder::ExtractNumeric
-
 static bool
 IsStillAlpha(const BitArray& bits, int pos)
 {
@@ -140,7 +125,7 @@ IsStillAlpha(const BitArray& bits, int pos)
 	}
 
 	// We now check if it's a valid 5-bit value (0..9 and FNC1)
-	int fiveBitValue = ExtractNumeric(bits, pos, 5);
+	int fiveBitValue = ToInt(bits, pos, 5);
 	if (fiveBitValue >= 5 && fiveBitValue < 16) {
 		return true;
 	}
@@ -149,7 +134,7 @@ IsStillAlpha(const BitArray& bits, int pos)
 		return false;
 	}
 
-	int sixBitValue = ExtractNumeric(bits, pos, 6);
+	int sixBitValue = ToInt(bits, pos, 6);
 	return sixBitValue >= 16 && sixBitValue < 63; // 63 not included
 }
 
@@ -160,7 +145,7 @@ IsStillIsoIec646(const BitArray& bits, int pos)
 		return false;
 	}
 
-	int fiveBitValue = ExtractNumeric(bits, pos, 5);
+	int fiveBitValue = ToInt(bits, pos, 5);
 	if (fiveBitValue >= 5 && fiveBitValue < 16) {
 		return true;
 	}
@@ -169,7 +154,7 @@ IsStillIsoIec646(const BitArray& bits, int pos)
 		return false;
 	}
 
-	int sevenBitValue = ExtractNumeric(bits, pos, 7);
+	int sevenBitValue = ToInt(bits, pos, 7);
 	if (sevenBitValue >= 64 && sevenBitValue < 116) {
 		return true;
 	}
@@ -178,7 +163,7 @@ IsStillIsoIec646(const BitArray& bits, int pos)
 		return false;
 	}
 
-	int eightBitValue = ExtractNumeric(bits, pos, 8);
+	int eightBitValue = ToInt(bits, pos, 8);
 	return eightBitValue >= 232 && eightBitValue < 253;
 }
 
@@ -202,7 +187,7 @@ IsStillNumeric(const BitArray& bits, int pos)
 static DecodedChar
 DecodeAlphanumeric(const BitArray& bits, int pos)
 {
-	int fiveBitValue = ExtractNumeric(bits, pos, 5);
+	int fiveBitValue = ToInt(bits, pos, 5);
 	if (fiveBitValue == 15) {
 		return DecodedChar(pos + 5, DecodedChar::FNC1);
 	}
@@ -211,7 +196,7 @@ DecodeAlphanumeric(const BitArray& bits, int pos)
 		return DecodedChar(pos + 5, (char)('0' + fiveBitValue - 5));
 	}
 
-	int sixBitValue = ExtractNumeric(bits, pos, 6);
+	int sixBitValue = ToInt(bits, pos, 6);
 
 	if (sixBitValue >= 32 && sixBitValue < 58) {
 		return DecodedChar(pos + 6, (char)(sixBitValue + 33));
@@ -287,6 +272,11 @@ ParseAlphaBlock(const BitArray& bits, ParsingState& state, std::string& buffer)
 		state.position = alpha.newPosition;
 
 		if (alpha.isFNC1()) {
+			// Allow for some generators incorrectly placing a numeric latch "000" after an FNC1
+			if (state.position + 7 < bits.size() && ToInt(bits, state.position, 7) < 8) {
+				state.position += 3;
+			}
+			state.encoding = ParsingState::NUMERIC; // FNC1 latches to numeric encodation
 			return DecodedInformation(state.position, buffer); //end of the char block
 		}
 		buffer.push_back(alpha.value);
@@ -311,7 +301,7 @@ ParseAlphaBlock(const BitArray& bits, ParsingState& state, std::string& buffer)
 static DecodedChar
 DecodeIsoIec646(const BitArray& bits, int pos)
 {
-	int fiveBitValue = ExtractNumeric(bits,pos, 5);
+	int fiveBitValue = ToInt(bits,pos, 5);
 	if (fiveBitValue == 15) {
 		return DecodedChar(pos + 5, DecodedChar::FNC1);
 	}
@@ -320,7 +310,7 @@ DecodeIsoIec646(const BitArray& bits, int pos)
 		return DecodedChar(pos + 5, (char)('0' + fiveBitValue - 5));
 	}
 
-	int sevenBitValue = ExtractNumeric(bits, pos, 7);
+	int sevenBitValue = ToInt(bits, pos, 7);
 
 	if (sevenBitValue >= 64 && sevenBitValue < 90) {
 		return DecodedChar(pos + 7, (char)(sevenBitValue + 1));
@@ -330,7 +320,7 @@ DecodeIsoIec646(const BitArray& bits, int pos)
 		return DecodedChar(pos + 7, (char)(sevenBitValue + 7));
 	}
 
-	int eightBitValue = ExtractNumeric(bits, pos, 8);
+	int eightBitValue = ToInt(bits, pos, 8);
 	if (eightBitValue < 232 || eightBitValue > 252)
 		throw std::runtime_error("Decoding invalid ISO-IEC-646 value");
 
@@ -347,6 +337,11 @@ ParseIsoIec646Block(const BitArray& bits, ParsingState& state, std::string& buff
 		DecodedChar iso = DecodeIsoIec646(bits, state.position);
 		state.position = iso.newPosition;
 		if (iso.isFNC1()) {
+			// Allow for some generators incorrectly placing a numeric latch "000" after an FNC1
+			if (state.position + 7 < bits.size() && ToInt(bits, state.position, 7) < 8) {
+				state.position += 3;
+			}
+			state.encoding = ParsingState::NUMERIC; // FNC1 latches to numeric encodation
 			return DecodedInformation(state.position, buffer);
 		}
 		buffer.push_back(iso.value);
@@ -372,13 +367,13 @@ static DecodedNumeric
 DecodeNumeric(const BitArray& bits, int pos)
 {
 	if (pos + 7 > bits.size()) {
-		int numeric = ExtractNumeric(bits, pos, 4);
+		int numeric = ToInt(bits, pos, 4);
 		if (numeric == 0) {
 			return DecodedNumeric(bits.size(), DecodedNumeric::FNC1, DecodedNumeric::FNC1);
 		}
 		return DecodedNumeric(bits.size(), numeric - 1, DecodedNumeric::FNC1);
 	}
-	int numeric = ExtractNumeric(bits, pos, 7);
+	int numeric = ToInt(bits, pos, 7);
 	int digit1 = (numeric - 8) / 11;
 	int digit2 = (numeric - 8) % 11;
 
@@ -390,6 +385,8 @@ ParseNumericBlock(const BitArray& bits, ParsingState& state, std::string& buffer
 {
 	while (IsStillNumeric(bits, state.position)) {
 		DecodedNumeric numeric = DecodeNumeric(bits, state.position);
+		if (!numeric.isValid())
+			break;
 		state.position = numeric.newPosition;
 
 		if (numeric.isFirstDigitFNC1()) {
@@ -447,13 +444,16 @@ DoDecodeGeneralPurposeField(ParsingState& state, const BitArray& bits, std::stri
 }
 
 DecodeStatus
-GenericAppIdDecoder::DecodeGeneralPurposeField(const BitArray& bits, int pos, std::string& result)
+DecodeAppIdGeneralPurposeField(const BitArray& bits, int& pos, int& remainingValue, std::string& result)
 {
 	try
 	{
 		ParsingState state;
 		state.position = pos;
-		result += DoDecodeGeneralPurposeField(state, bits, std::string()).newString;
+		DecodedInformation info = DoDecodeGeneralPurposeField(state, bits, std::string());
+		result += info.newString;
+		pos = state.position;
+		remainingValue = info.remainingValue;
 		return DecodeStatus::NoError;
 	}
 	catch (const std::exception &)
@@ -463,19 +463,26 @@ GenericAppIdDecoder::DecodeGeneralPurposeField(const BitArray& bits, int pos, st
 }
 
 DecodeStatus
-GenericAppIdDecoder::DecodeAllCodes(const BitArray& bits, int pos, std::string& result)
+DecodeAppIdAllCodes(const BitArray& bits, int pos, int remainingValue, std::string& result)
 {
 	try
 	{
 		ParsingState state;
 		std::string remaining;
+		if (remainingValue != -1) {
+			remaining = std::to_string(remainingValue);
+		}
 		while (true) {
 			state.position = pos;
 			DecodedInformation info = DoDecodeGeneralPurposeField(state, bits, remaining);
 			std::string parsedFields;
-			auto status = FieldParser::ParseFieldsInGeneralPurpose(info.newString, parsedFields);
+			auto status = ParseFieldsInGeneralPurpose(info.newString, parsedFields);
 			if (StatusIsError(status)) {
-				return status;
+				if (result.empty() && remaining.empty()){
+					result = info.newString;
+					return DecodeStatus::NoError;
+				} else
+					return status;
 			}
 			result += parsedFields;
 			if (info.isRemaining()) {
@@ -498,6 +505,4 @@ GenericAppIdDecoder::DecodeAllCodes(const BitArray& bits, int pos, std::string&
 	return DecodeStatus::FormatError;
 }
 
-} // RSS
-} // OneD
-} // ZXing
+} // namespace ZXing::OneD::DataBar
