@@ -34,6 +34,7 @@
 #include <map>
 #include <optional>
 #include <set>
+#include <sstream>
 #include <string>
 #include <string_view>
 #include <vector>
@@ -72,6 +73,65 @@ namespace {
 	};
 }
 
+// Helper for `compareResult()` - map `key` to Result property, converting value to std::string
+static std::string getResultValue(const Result& result, const std::string& key)
+{
+	if (key == "ecLevel")
+		return TextUtfEncoding::ToUtf8(result.ecLevel());
+	if (key == "orientation")
+		return std::to_string(result.orientation());
+	if (key == "numBits")
+		return std::to_string(result.numBits());
+	if (key == "symbologyIdentifier")
+		return result.symbologyIdentifier();
+	if (key == "sequenceSize")
+		return std::to_string(result.sequenceSize());
+	if (key == "sequenceIndex")
+		return std::to_string(result.sequenceIndex());
+	if (key == "sequenceId")
+		return result.sequenceId();
+	if (key == "isLastInSequence")
+		return result.isLastInSequence() ? "true" : "false";
+	if (key == "isPartOfSequence")
+		return result.isPartOfSequence() ? "true" : "false";
+	if (key == "isMirrored")
+		return result.isMirrored() ? "true" : "false";
+	if (key == "readerInit")
+		return result.readerInit() ? "true" : "false";
+
+	return fmt::format("***Unknown key '{}'***", key);
+}
+
+// Read ".result.txt" file contents `expected` with lines "key=value" and compare to `actual`
+static bool compareResult(const Result& result, const std::string& expected, std::string& actual)
+{
+	bool ret = true;
+
+	actual.clear();
+	actual.reserve(expected.size());
+
+	std::stringstream expectedLines(expected);
+	std::string expectedLine;
+	while (std::getline(expectedLines, expectedLine)) {
+		if (expectedLine.empty() || expectedLine[0] == '#')
+			continue;
+		auto equals = expectedLine.find('=');
+		if (equals == std::string::npos) {
+			actual += "***Bad format, missing equals***\n";
+			return false;
+		}
+		std::string key = expectedLine.substr(0, equals);
+		std::string expectedValue = expectedLine.substr(equals + 1);
+		std::string actualValue = getResultValue(result, key);
+		if (actualValue != expectedValue) {
+			ret = false;
+			actualValue += " ***Mismatch***";
+		}
+		actual += key + '=' + actualValue + '\n';
+	}
+	return ret;
+}
+
 static std::string checkResult(const fs::path& imgPath, std::string_view expectedFormat, const Result& result)
 {
 	if (auto format = ToString(result.format()); expectedFormat != format)
@@ -82,6 +142,12 @@ static std::string checkResult(const fs::path& imgPath, std::string_view expecte
 		return ifs ? std::optional(std::string(std::istreambuf_iterator<char>(ifs), std::istreambuf_iterator<char>())) : std::nullopt;
 	};
 
+	if (auto expected = readFile(".result.txt")) {
+		std::string actual;
+		if (!compareResult(result, *expected, actual))
+			return fmt::format("Result mismatch: expected\n{} but got\n{}", *expected, actual);
+	}
+
 	if (auto expected = readFile(".txt")) {
 		auto utf8Result = TextUtfEncoding::ToUtf8(result.text());
 		return utf8Result != *expected ? fmt::format("Content mismatch: expected '{}' but got '{}'", *expected, utf8Result) : "";
@@ -191,15 +257,13 @@ static void doRunTests(
 	}
 }
 
-static auto readPDF417s = [](const BinaryBitmap& image) { return Pdf417::Reader({}).decodeMultiple(image); };
-static auto readQRCodes = [](const BinaryBitmap& image) { return std::list{QRCode::Reader({}).decode(image)}; };
-
-template<typename READER>
-static Result readMultiple(const std::vector<fs::path>& imgPaths, int rotation, READER read)
+static Result readMultiple(const std::vector<fs::path>& imgPaths, std::string_view format)
 {
 	std::list<Result> allResults;
 	for (const auto& imgPath : imgPaths) {
-		auto results = read(ThresholdBinarizer(ImageLoader::load(imgPath), 127));
+		auto results =
+			ReadBarcodes(ImageLoader::load(imgPath),
+						 DecodeHints().setFormats(BarcodeFormatFromString(format.data())).setTryDownscale(false));
 		allResults.insert(allResults.end(), results.begin(), results.end());
 	}
 
@@ -217,7 +281,10 @@ static Result readMultiple(const std::vector<fs::path>& imgPaths, int rotation,
 	for (const auto& r : allResults)
 		text.append(r.text());
 
-	return {std::move(text), {}, allResults.front().format()};
+	const auto& first = allResults.front();
+	StructuredAppendInfo sai{ first.sequenceIndex(), first.sequenceSize(), first.sequenceId() };
+	return Result(std::move(text), {}, first.format(), std::string(first.symbologyIdentifier()), {}, std::move(sai),
+				  first.readerInit());
 }
 
 static void doRunStructuredAppendTest(
@@ -243,8 +310,7 @@ static void doRunStructuredAppendTest(
 		auto startTime = std::chrono::steady_clock::now();
 
 		for (const auto& [testPath, testImgPaths] : imageGroups) {
-			auto result = format == "QRCode" ? readMultiple(testImgPaths, test.rotation, readQRCodes)
-											 : readMultiple(testImgPaths, test.rotation, readPDF417s);
+			auto result = readMultiple(testImgPaths, format);
 			if (result.isValid()) {
 				auto error = checkResult(testPath, format, result);
 				if (!error.empty())
@@ -283,12 +349,12 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		auto startTime = std::chrono::steady_clock::now();
 
 		// clang-format off
-		runTests("aztec-1", "Aztec", 21, {
-			{ 20, 20, 0   },
-			{ 20, 20, 90  },
-			{ 20, 20, 180 },
-			{ 20, 20, 270 },
-			{ 21, 0, pure },
+		runTests("aztec-1", "Aztec", 22, {
+			{ 21, 21, 0   },
+			{ 21, 21, 90  },
+			{ 21, 21, 180 },
+			{ 21, 21, 270 },
+			{ 22, 0, pure },
 		});
 
 		runTests("aztec-2", "Aztec", 22, {
@@ -298,12 +364,12 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 			{ 3, 3, 270 },
 		});
 
-		runTests("datamatrix-1", "DataMatrix", 25, {
-			{ 25, 25, 0   },
-			{  0, 25, 90  },
-			{  0, 25, 180 },
-			{  0, 25, 270 },
-			{ 24, 0, pure },
+		runTests("datamatrix-1", "DataMatrix", 26, {
+			{ 26, 26, 0   },
+			{  0, 26, 90  },
+			{  0, 26, 180 },
+			{  0, 26, 270 },
+			{ 25, 0, pure },
 		});
 
 		runTests("datamatrix-2", "DataMatrix", 13, {
@@ -320,12 +386,12 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 			{  0, 19, 270 },
 		});
 
-		runTests("datamatrix-4", "DataMatrix", 20, {
-			{ 20, 20, 0   },
-			{  0, 20, 90  },
-			{  0, 20, 180 },
-			{  0, 20, 270 },
-			{ 18, 0, pure },
+		runTests("datamatrix-4", "DataMatrix", 21, {
+			{ 21, 21, 0   },
+			{  0, 21, 90  },
+			{  0, 21, 180 },
+			{  0, 21, 270 },
+			{ 19, 0, pure },
 		});
 
 		runTests("codabar-1", "Codabar", 11, {
@@ -334,8 +400,8 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		});
 
 		runTests("codabar-2", "Codabar", 4, {
-			{ 3, 3, 0   },
-			{ 3, 3, 180 },
+			{ 2, 3, 0   },
+			{ 2, 3, 180 },
 		});
 
 		runTests("code39-1", "Code39", 4, {
@@ -364,7 +430,7 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		});
 
 		runTests("code128-2", "Code128", 21, {
-			{ 19, 21, 0   },
+			{ 18, 21, 0   },
 			{ 19, 21, 180 },
 		});
 
@@ -376,10 +442,11 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		runTests("ean8-1", "EAN-8", 8, {
 			{ 8, 8, 0   },
 			{ 8, 8, 180 },
+			{ 7, 0, pure },
 		});
 
 		runTests("ean13-1", "EAN-13", 31, {
-			{ 26, 29, 0   },
+			{ 24, 29, 0   },
 			{ 23, 29, 180 },
 		});
 
@@ -394,18 +461,18 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		});
 
 		runTests("ean13-4", "EAN-13", 22, {
-			{ 7, 14, 0   },
-			{ 8, 14, 180 },
+			{ 6, 13, 0   },
+			{ 8, 13, 180 },
 		});
 
 		runTests("ean13-extension-1", "EAN-13", 5, {
-			{ 4, 5, 0 },
+			{ 3, 5, 0 },
 			{ 3, 5, 180 },
 		}, DecodeHints().setEanAddOnSymbol(EanAddOnSymbol::Require));
 
-		runTests("itf-1", "ITF", 10, {
-			{ 10, 10, 0   },
-			{ 10, 10, 180 },
+		runTests("itf-1", "ITF", 11, {
+			{ 10, 11, 0   },
+			{ 10, 11, 180 },
 		});
 
 		runTests("itf-2", "ITF", 6, {
@@ -422,48 +489,49 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		});
 
 		runTests("upca-1", "UPC-A", 12, {
-			{  9, 12, 0, 1, 0   },
-			{ 11, 12, 0, 1, 180 },
+			{  9, 12, 0   },
+			{ 11, 12, 180 },
 		});
 
 		runTests("upca-2", "UPC-A", 36, {
 			{ 17, 22, 0   },
-			{ 18, 22, 180 },
+			{ 16, 22, 180 },
 		});
 
 		runTests("upca-3", "UPC-A", 21, {
-			{ 7, 10, 0, 1, 0   },
-			{ 8, 10, 0, 1, 180 },
+			{ 7, 10, 0   },
+			{ 8, 10, 180 },
 		});
 
 		runTests("upca-4", "UPC-A", 19, {
-			{ 9, 11, 0, 1, 0   },
-			{ 9, 11, 0, 1, 180 },
+			{ 8, 12, 0   },
+			{ 9, 12, 180 },
 		});
 
 		runTests("upca-5", "UPC-A", 32, {
 			{ 17, 20, 0   },
-			{ 19, 20, 180 },
+			{ 18, 20, 180 },
 		});
 		
 		runTests("upca-extension-1", "UPC-A", 6, {
-			{ 3, 6, 0 },
-			{ 4, 6, 180 },
+			{ 4, 4, 0 },
+			{ 3, 4, 180 },
 		}, DecodeHints().setEanAddOnSymbol(EanAddOnSymbol::Require));
 
 		runTests("upce-1", "UPC-E", 3, {
 			{ 3, 3, 0   },
 			{ 3, 3, 180 },
+			{ 3, 0, pure },
 		});
 
 		runTests("upce-2", "UPC-E", 28, {
-			{ 19, 22, 0, 1, 0   },
+			{ 17, 22, 0, 1, 0   },
 			{ 20, 22, 1, 1, 180 },
 		});
 
 		runTests("upce-3", "UPC-E", 11, {
-			{ 6, 8, 0   },
-			{ 6, 8, 180 },
+			{ 5, 7, 0   },
+			{ 6, 7, 180 },
 		});
 
 		runTests("rss14-1", "DataBar", 6, {
@@ -472,13 +540,14 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		});
 
 		runTests("rss14-2", "DataBar", 16, {
-			{ 8 , 10, 0   },
-			{ 10, 10, 180 },
+			{ 8, 10, 0   },
+			{ 9, 10, 180 },
 		});
 
 		runTests("rssexpanded-1", "DataBarExpanded", 33, {
 			{ 33, 33, 0   },
 			{ 33, 33, 180 },
+			{ 33, 0, pure },
 		});
 
 		runTests("rssexpanded-2", "DataBarExpanded", 15, {
@@ -489,11 +558,13 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 		runTests("rssexpanded-3", "DataBarExpanded", 118, {
 			{ 118, 118, 0   },
 			{ 118, 118, 180 },
+			{ 118, 0, pure },
 		});
 
 		runTests("rssexpandedstacked-1", "DataBarExpanded", 65, {
 			{ 60, 65, 0   },
 			{ 60, 65, 180 },
+			{ 60, 0, pure },
 		});
 
 		runTests("rssexpandedstacked-2", "DataBarExpanded", 7, {
@@ -508,12 +579,12 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 			{ 16, 16, 270 },
 		});
 
-		runTests("qrcode-2", "QRCode", 40, {
-			{ 38, 38, 0   },
-			{ 38, 38, 90  },
-			{ 38, 38, 180 },
-			{ 38, 38, 270 },
-			{ 20, 1, pure }, // the misread is the 'outer' symbol in 16.png
+		runTests("qrcode-2", "QRCode", 46, {
+			{ 44, 44, 0   },
+			{ 44, 44, 90  },
+			{ 44, 44, 180 },
+			{ 44, 44, 270 },
+			{ 21, 1, pure }, // the misread is the 'outer' symbol in 16.png
 		});
 
 		runTests("qrcode-3", "QRCode", 28, {
@@ -572,7 +643,7 @@ int runBlackBoxTests(const fs::path& testPathPrefix, const std::set<std::string>
 			{ 2, 2, 0   },
 		});
 
-		runTests("falsepositives-1", "None", 24, {
+		runTests("falsepositives-1", "None", 25, {
 			{ 0, 0, 0, 0, 0   },
 			{ 0, 0, 0, 0, 90  },
 			{ 0, 0, 0, 0, 180 },
