@@ -14,29 +14,32 @@
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
-#include "gtest/gtest.h"
+
 #include "aztec/AZHighLevelEncoder.h"
 #include "BitArray.h"
 #include "BitArrayUtility.h"
+#include "StructuredAppend.h"
+#include "TextDecoder.h"
 
+#include "gtest/gtest.h"
 #include <algorithm>
 
-namespace ZXing {
-	namespace Aztec {
-		std::string GetEncodedData(const std::vector<bool>& correctedBits);
-	}
+namespace ZXing::Aztec {
+
+struct AztecData
+{
+	std::wstring text;
+	std::string symbologyIdentifier;
+	StructuredAppendInfo sai;
+};
+
+AztecData GetEncodedData(const BitArray& bits, const std::string& characterSet = "");
+
 }
 
 using namespace ZXing;
 
 namespace {
-	
-	std::vector<bool> ToBoolArray(const BitArray& arr) {
-		std::vector<bool> result(arr.size());
-		std::copy_n(arr.begin(), arr.size(), result.begin());
-		return result;
-	}
-
 	std::string StripSpaces(std::string str) {
 		str.erase(std::remove_if(str.begin(), str.end(), isspace), str.end());
 		return str;
@@ -45,14 +48,16 @@ namespace {
 	void TestHighLevelEncodeString(const std::string& s, const std::string& expectedBits) {
 		BitArray bits = Aztec::HighLevelEncoder::Encode(s);
 		EXPECT_EQ(Utility::ToString(bits), StripSpaces(expectedBits)) << "highLevelEncode() failed for input string: " + s;
-		EXPECT_EQ(s, Aztec::GetEncodedData(ToBoolArray(bits)));
+		EXPECT_EQ(TextDecoder::FromLatin1(s), Aztec::GetEncodedData(bits).text);
 	}
 
 	void TestHighLevelEncodeString(const std::string& s, int expectedReceivedBits) {
 		BitArray bits = Aztec::HighLevelEncoder::Encode(s);
 		int receivedBitCount = Size(Utility::ToString(bits));
 		EXPECT_EQ(receivedBitCount, expectedReceivedBits) << "highLevelEncode() failed for input string: " + s;
-		EXPECT_EQ(s, Aztec::GetEncodedData(ToBoolArray(bits)));
+		std::string symbologyIdentifier;
+		StructuredAppendInfo sai;
+		EXPECT_EQ(TextDecoder::FromLatin1(s), Aztec::GetEncodedData(bits).text);
 	}
 }
 
